local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")


function SliceCommander:ArrayPos()
	local returnArray = {}
	for key = -15, 15, 1 do
		returnArray[key] = key
	end
	return returnArray
end

local SC_OptionsTable_Sound = {
	name = L["Sound settings"],
	handler = SliceCommander,
	type = "group",
	args = {--Sound settings
		headerSndSoundSetting = {--Slice and Dice
			order = 1,
			name = L["Slice and Dice"],
			type = "header"
		},
		Tick3 = {--Alert - 3 sec
			order = 2,
			name = L["Alert - 3 sec"],
			desc = L["Alert 3 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Tick")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Tick3 = val
			end,
			get = function(info) return SliceCommander_Save.Tick3 end
		},
		Tick2 = {--Alert - 2 sec
			order = 4,
			name = L["Alert - 2 sec"],
			desc = L["Alert 2 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Tick")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Tick2 = val
			end,
			get = function(info) return SliceCommander_Save.Tick2 end
		},
		Tick1 = {--Alert - 1 sec
			order = 6,
			name = L["Alert - 1 sec"],
			desc = L["Alert 1 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Tick")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Tick1 = val
			end,
			get = function(info) return SliceCommander_Save.Tick1 end
		},
		Apply3 = {--Refresh - 3 sec
			order = 3,
			name = L["Refresh - 3 sec"],
			desc = L["Refresh 3 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Apply3 = val
			end,
			get = function(info) return SliceCommander_Save.Apply3 end
		},
		Apply2 = {--Refresh - 2 sec
			order = 5,
			name = L["Refresh - 2 sec"],
			desc = L["Refresh 2 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Apply2 = val
			end,
			get = function(info) return SliceCommander_Save.Apply2 end
		},
		Apply1 = {--Refresh - 1 sec
			order = 7,
			name = L["Refresh - 1 sec"],
			desc = L["Refresh 1 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Apply1 = val
			end,
			get = function(info) return SliceCommander_Save.Apply1 end
		},
		Fail = {--Faillure
			order = 8,
			name = L["Faillure"],
			desc = L["The refresh failled"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Fail")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Fail = val
			end,
			get = function(info) return SliceCommander_Save.Fail end
		},

		Applied = {--Applied
			order = 9,
			name = L["Applied"],
			desc = L["Slice and Dice is applied"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Applied = val
			end,
			get = function(info) return SliceCommander_Save.Applied end
		},
		Expire = {--Expire
			order = 10,
			name = L["Expired"],
			desc = L["Slice and Dice buff expire"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Fail")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.Expire = val
			end,
			get = function(info) return SliceCommander_Save.Expire end
		},
		headerRupSoundSetting = {--Rupture
			order = 11,
			name = L["Rupture"],
			type = "header"
		},
		RUP_Alert = {--Alert
			order = 12,
			name = L["Alert"],
			desc = L["Alert 3, 2 and 1 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Tick")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Alert = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Alert end
		},
		RUP_Refresh3 = {--Refresh - 3 sec
			order = 13,
			name = L["Alert - 3 sec"],
			desc = L["Alert 3 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Refresh3 = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Refresh3 end
		},
		RUP_Refresh2 = {--Refresh - 2 sec
			order = 15,
			name = L["Alert - 2 sec"],
			desc = L["Alert 2 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Refresh2 = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Refresh2 end
		},
		RUP_Refresh1 = {--Refresh - 1 sec
			order = 17,
			name = L["Alert - 1 sec"],
			desc = L["Alert 1 secondes before fade"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Refresh1 = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Refresh1 end
		},
		RUP_Fail = {--Faillure
			order = 14,
			name = L["Faillure"],
			desc = L["The refresh failled"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Fail")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Fail = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Fail end
		},
		RUP_Applied = {--Applied
			order = 16,
			name = L["Applied"],
			desc = L["Rupture is applied"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Apply")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Applied = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Applied end
		},
		RUP_Expire = {--Expire
			order = 18,
			name = L["Expired"],
			desc = L["Rupture debuff expire"],
			type = "select",
			values = function()
				return SliceCommander:ConfigSoundMenuInit("Fail")
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.RUP_Expire = val
			end,
			get = function(info) return SliceCommander_Save.RUP_Expire end
		},
	}
}

local SC_OptionsTable_DisplayTimer = {
	name = L["Display settings"],
	handler = SliceCommander,
	type = "group",
	args = {--Display setting
		displayDescription = {
			order = 0,
			name = L["displayDescription"],
			fontSize = "medium",
			type = "description"
		},
		headerEnergySetting = {--Combo Point Settings
			order = 1,
			name = L["Energy bar Settings"],
			type = "header"
		},
		HideEnergy = {--Hide Energy
			order = 2,
			name = L["Hide Energy bar"],
			desc = L["Hide Energy bar"],
			type = "toggle",
			set = function(info,val)
				if val then
					VTimerEnergy:Hide()
				else
					VTimerEnergy:Show()
				end
				SliceCommander_Save.HideEnergy = val
			end,
			get = function(info) return SliceCommander_Save.HideEnergy end
		},
		ShowBG = {--Show Bandit's Guile
			order = 3,
			name = L["Show Bandit's Guile"],
			desc = L["Show Bandit's Guile"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.BanditGuild = val
			end,
			get = function(info) return SliceCommander_Save.BanditGuild end
		},
		BGSound = {--Bandit's Guile Sound
			order = 4,
			name = L["Bandit's Guile Sound"],
			desc = L["Bandit's Guile Sound. It has no effect if Bandit's Guile is not display."],
			type = "select",
			values = function()
				local returnArray = { ["None"] = L["None"] }
				for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
					returnArray[SoundName] = SoundName
				end
				return returnArray
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.BGSound = val
			end,
			get = function(info)
				return SliceCommander_Save.BGSound
			end
		},
		EnergyTrans = {--Energy bar transparency
			order = 5,
			name = L["Transparency (Full)"],
			desc = L["The fade out value of the energy bar when it is full."],
			type = "range",
			min = 0,
			max = 100,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				SliceCommander:SetPoisonTransparency(val)
				SliceCommander_Save.EnergyTrans = val
			end,
			get = function(info) return SliceCommander_Save.EnergyTrans end,
			isPercent = false
		},
		Energy1 = {--Energy Marker 1
			order = 6,
			name = L["Energy Marker 1"],
			type = "range",
			min = 0,
			max = 120,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				if val > UnitManaMax("player") then
					val = UnitManaMax("player")
				end
				SliceCommander_Save.Energy1 = val
				SliceCommander:Config_OtherVars()
			end,
			get = function(info) return SliceCommander_Save.Energy1 end,
			isPercent = false
		},
		Energy2 = {--Energy Marker 2
			order = 7,
			name = L["Energy Marker 2"],
			type = "range",
			min = 0,
			max = 120,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				if val > UnitManaMax("player") then
					val = UnitManaMax("player")
				end
				SliceCommander_Save.Energy2 = val
				SliceCommander:Config_OtherVars()
			end,
			get = function(info) return SliceCommander_Save.Energy2 end,
			isPercent = false
		},
		Energy3 = {--Energy Marker 3
			order = 8,
			name = L["Energy Marker 3"],
			type = "range",
			min = 0,
			max = 120,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				if val > UnitManaMax("player") then
					val = UnitManaMax("player")
				end
				SliceCommander_Save.Energy3 = val
				SliceCommander:Config_OtherVars()
			end,
			get = function(info) return SliceCommander_Save.Energy3 end,
			isPercent = false
		},
		headerHealthSetting = {--Health Settings
			order = 10,
			name = L["Health bar Settings"],
			type = "header"
		},
		HEALTHBarShow = {--Show Health
			order = 11,
			name = L["Show Health player bar"],
			desc = L["Show Health player bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.HEALTH = val
			end,
			get = function(info) return SliceCommander_Save.display.HEALTH end
		},
		PosHEALTH = {--Health position
			order = 12,
			name = L["Health bar position"],
			desc = L["The place where to display the Health bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.HEALTH = val
			end,
			get = function(info) return SliceCommander_Save.Pos.HEALTH end
		},
		MidHealthUnder = {
			order = 13,
			name = L["Sound under 50% health"],
			desc = L["The sound play when your health reach 50% health"],
			type = "select",
			values = function()
				local returnArray = { ["None"] = L["None"] }
				for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
					returnArray[SoundName] = SoundName
				end
				return returnArray
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.MidHealthUnder = val
			end,
			get = function(info)
				return SliceCommander_Save.MidHealthUnder
			end
		},
		LowHealthUnder = {
			order = 14,
			name = L["Sound under 35% health"],
			desc = L["The sound play when your health reach 35% health"],
			type = "select",
			values = function()
				local returnArray = { ["None"] = L["None"] }
				for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
					returnArray[SoundName] = SoundName
				end
				return returnArray
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.LowHealthUnder = val
			end,
			get = function(info)
				return SliceCommander_Save.LowHealthUnder
			end
		},
		healthPercentage = {
			order = 15,
			name = L["Show HP Value"],
			desc = L["Show HP value rather than health percentage."],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.healthValue = val
				SliceCommander:PlayerHealth()
			end,
			get = function(info) return SliceCommander_Save.healthValue end
		},
		healthCombat = {
			order = 16,
			name = L["Hide out of combat"],
			desc = L["Hide health bar when you are not in combat."],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.healthHideCombat = val
				SliceCommander:PlayerHealth()
			end,
			get = function(info) return SliceCommander_Save.healthHideCombat end
		},
		headerDPSetting = {--Poison bar Settings
			order = 28,
			name = L["Poison bar Settings"],
			type = "header"
		},
		DPBarShow = {--Show Poison
			order = 29,
			name = L["Show Poison bar"],
			desc = L["Display the Poison timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.DP = val
			end,
			get = function(info) return SliceCommander_Save.display.DP end
		},
		PosDP = {--Poison position
			order =30,
			name = L["Poison bar position"],
			desc = L["The place where to display the Poison bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.DP = val
			end,
			get = function(info) return SliceCommander_Save.Pos.DP end
		},
		EnvenomShow = {--Show Envenom
			order = 31,
			name = L["Show envenom buff"],
			desc = L["The timer bar in default will display the timer of deadly poison on the target."],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.ENV = val
			end,
			get = function(info) return SliceCommander_Save.display.ENV end
		},
		PosENV = {--Poison position
			order = 32,
			name = L["Envenom bar position"],
			desc = L["The place where to display the Envenom bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.ENV = val
			end,
			get = function(info) return SliceCommander_Save.Pos.ENV end
		},
		PoisonShow = {--Show Poison weapon enchant
			order = 33,
			name = L["Show poison weapon enchant"],
			desc = L["Display two thin poison bar timer on the energy bar corresponding of each temporary enchant on each weapon."],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.Poison = val
			end,
			get = function(info) return SliceCommander_Save.display.Poison end
		},
		PoisonTransparencyShow = {--Show Poison weapon enchant
			order = 34,
			name = L["Disable energy bar fade out"],
			desc = L["Disable energy bar and poison bar fade out when poison on weapon(s) is up"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.PoisonTransparencyShow = val
			end,
			get = function(info) return SliceCommander_Save.PoisonTransparencyShow end
		},
		headerVendettaSetting = {--Vendetta Settings
			order = 101,
			name = L["Vendetta Settings"],
			type = "header"
		},
		VenBarShow = {--Show vendetta
			order = 102,
			name = L["Show vendetta bar"],
			desc = L["Display the vendetta timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.VEN = val
				SliceCommander_Save.display.VENCD = val
			end,
			get = function(info) return SliceCommander_Save.display.VEN end
		},
		PosVEN = {--Vendetta position
			order = 103,
			name = L["Vendetta bar position"],
			desc = L["The place where to display the vendetta bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.VEN = val
				SliceCommander_Save.Pos.VENCD = val
			end,
			get = function(info) return SliceCommander_Save.Pos.VEN end
		},
		headerToTSetting = {--Tricks of the Trade Settings
			order = 104,
			name = L["Tricks of the Trade Settings"],
			type = "header"
		},
		TotTBarShow = {--Show ToT
			order = 105,
			name = L["Show Tricks of the Trade bar"],
			desc = L["Display the Tricks of the Trade timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.ToT = val
			end,
			get = function(info) return SliceCommander_Save.display.ToT end
		},
		PosToT = {--ToT position
			order = 106,
			name = L["Tricks of the Trade bar position"],
			desc = L["The place where to display the Tricks of the Trade bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.ToT = val
			end,
			get = function(info) return SliceCommander_Save.Pos.ToT end
		},
		ToTTarget = {
			order = 107,
			name = L["Taget name"],
			desc = L["Write a macro which Tricks the target if it is friendly, if not the target of target. Except if a modifier is down the Tricks will target the player name specify here."],
			type = "input",
			set = function(info,val)
				SliceCommander_Save.ToTTarget = val
				SliceCommander:CreateToTMacro()
			end,
			get = function(info) return SliceCommander_Save.ToTTarget end
		},
		headerRecSetting = {--Recuperate Settings
			order = 110,
			name = L["Recuperate Settings"],
			type = "header"
		},
		RECBarShow = {--Show Recuperate
			order = 111,
			name = L["Show Recuperate bar"],
			desc = L["Display the Recuperate timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.REC = val
			end,
			get = function(info) return SliceCommander_Save.display.REC end
		},
		PosREC = {--Recuperate position
			order = 112,
			name = L["Recuperate bar position"],
			desc = L["The place where to display the Recuperate bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.REC = val
			end,
			get = function(info) return SliceCommander_Save.Pos.REC end
		},
		headerKSSetting = {--Stun bar Settings
			order = 113,
			name = L["Kidney shot bar Settings"],
			type = "header"
		},
		KSBarShow = {--Show Stun
			order = 114,
			name = L["Kidney shot bar"],
			desc = L["Display the Kidney shot timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.KS = val
			end,
			get = function(info) return SliceCommander_Save.display.KS end
		},
		PosKS = {--Stun position
			order = 115,
			name = L["Kidney shot bar position"],
			desc = L["The place where to display the Kidney shot bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.KS = val
			end,
			get = function(info) return SliceCommander_Save.Pos.KS end
		},
		headerCSSetting = {--Stun bar Settings
			order = 116,
			name = L["Cheap shot bar Settings"],
			type = "header"
		},
		CSBarShow = {--Show Stun
			order = 117,
			name = L["Show Cheap shot bar"],
			desc = L["Display the Cheap shot timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.CS = val
			end,
			get = function(info) return SliceCommander_Save.display.CS end
		},
		PosCS = {--Stun position
			order = 118,
			name = L["Cheap shot bar position"],
			desc = L["The place where to display the Cheap shot bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.CS = val
			end,
			get = function(info) return SliceCommander_Save.Pos.CS end
		},
		headerRupSetting = {--Blood Settings
			order = 121,
			name = L["Rupture bar Settings"],
			type = "header"
		},
		RupBarShow = {--Show Blood
			order = 122,
			name = L["Show Rupture bar"],
			desc = L["Display the Blood timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.RUP = val
			end,
			get = function(info) return SliceCommander_Save.display.RUP end
		},
		PosRUP = {--Blood position
			order = 123,
			name = L["Rupture bar position"],
			desc = L["The place where to display the Rupture bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.RUP = val
			end,
			get = function(info) return SliceCommander_Save.Pos.RUP end
		},
		headerGarSetting = {--Blood Settings
			order = 124,
			name = L["Garrote bar Settings"],
			type = "header"
		},
		GarBarShow = {--Show Blood
			order = 125,
			name = L["Show Garrote bar"],
			desc = L["Display the Garrote timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.GAR = val
			end,
			get = function(info) return SliceCommander_Save.display.GAR end
		},
		PosGAR = {--Blood position
			order = 126,
			name = L["Garrote bar position"],
			desc = L["The place where to display the Garrote bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.GAR = val
			end,
			get = function(info) return SliceCommander_Save.Pos.GAR end
		},
		headerSndSetting = {--Slice and Dice Settings
			order = 127,
			name = L["Slice and Dice Settings"],
			type = "header"
		},
		SnDBarShow = {--Show Slice and Dice
			order = 128,
			name = L["Show Slice and Dice bar"],
			desc = L["Display the Slice and Dice timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.SnD = val
			end,
			get = function(info) return SliceCommander_Save.display.SnD end
		},
		PosSnD = {--Slice and Dice position
			order = 129,
			name = L["Slice and Dice bar position"],
			desc = L["The place where to display the Slice and Dice bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.SnD = val
			end,
			get = function(info) return SliceCommander_Save.Pos.SnD end
		},
		headerCPSetting = {--Combo Point Settings
			order = 130,
			name = L["Combo Point bar Settings"],
			type = "header"
		},
		CPBarShow = {--Show Combo Point
			order = 131,
			name = L["Show Combo Point bar"],
			desc = L["Display the Combo Point timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.CP = val
			end,
			get = function(info) return SliceCommander_Save.display.CP end
		},
		PosCP = {--Combo Point position
			order = 132,
			name = L["Combo Point bar position"],
			desc = L["The place where to display the Combo Point bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.CP = val
			end,
			get = function(info) return SliceCommander_Save.Pos.CP end
		},
		headerThreatSetting = {--Threat Settings
			order = 133,
			name = L["Threat bar Settings"],
			type = "header"
		},
		THREATBarShow = {--Show Threat
			order = 134,
			name = L["Show Threat bar"],
			desc = L["Display the Threat bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.THREAT = val
			end,
			get = function(info) return SliceCommander_Save.display.THREAT end
		},
		PosTHREAT = {--Threat position
			order = 135,
			name = L["Threat bar position"],
			desc = L["The place where to display the Threat bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.THREAT = val
			end,
			get = function(info) return SliceCommander_Save.Pos.THREAT end
		},
		ThreatGroupOnly = {--Show Threat bar only in group
			order = 136,
			name = L["Threat bar only in group"],
			desc = L["Display the Threat bar only if you are in a group or a raid"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.ThreatGroupOnly = val
			end,
			get = function(info) return SliceCommander_Save.ThreatGroupOnly end
		},
		headerARSetting = {--Adrenaline rush Settings
			order = 137,
			name = L["Adrenaline Rush Settings"],
			type = "header"
		},
		ARBarShow = {--Show Adrenaline rush
			order = 138,
			name = L["Show Adrenaline Rush bar"],
			desc = L["Display the Adrenaline rush timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.AR = val
			end,
			get = function(info) return SliceCommander_Save.display.AR end
		},
		PosAR = {--Adrenaline rush position
			order = 139,
			name = L["Adrenaline rush bar position"],
			desc = L["The place where to display the Adrenaline rush bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.AR = val
				SliceCommander_Save.Pos.ARCD = val
			end,
			get = function(info) return SliceCommander_Save.Pos.AR end
		},
		ARCDBarShow = {--Show Adrenaline rush CD
			order = 140,
			name = L["Show Adrenaline Rush bar CD"],
			desc = L["Display the Adrenaline rush cooldown bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.ARCD = val
			end,
			get = function(info) return SliceCommander_Save.display.ARCD end
		},
		ARMarioMod = {--Enable Mario mod
			order = 141,
			name = L["Enable Mario mod"],
			desc = L["Enable mario star music during Adrenaline rush buff"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.ARMarioMod = val
			end,
			get = function(info) return SliceCommander_Save.ARMarioMod end
		},
		headerEASetting = {--Expose Armor Settings
			order = 151,
			name = L["Expose Armor Settings"],
			type = "header"
		},
		EABarShow = {--Show EA
			order = 152,
			name = L["Show Expose Armor bar"],
			desc = L["Display the Expose Armor timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.EA = val
			end,
			get = function(info) return SliceCommander_Save.display.EA end
		},
		PosEA = {--EA position
			order = 153,
			name = L["Expose Armor bar position"],
			desc = L["The place where to display the Expose Armor bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.EA = val
			end,
			get = function(info) return SliceCommander_Save.Pos.EA end
		},
		headerFESetting = {--Feint Settings
			order = 161,
			name = L["Feint Settings"],
			type = "header"
		},
		FEBarShow = {--Show Feint
			order = 162,
			name = L["Show Feint bar"],
			desc = L["Display the Feint timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.FE = val
			end,
			get = function(info) return SliceCommander_Save.display.FE end
		},
		PosFE = {--Feint position
			order = 163,
			name = L["Feint bar position"],
			desc = L["The place where to display the Feint bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.FE = val
				SliceCommander_Save.Pos.FECD = val
			end,
			get = function(info) return SliceCommander_Save.Pos.FE end
		},
		headerRSSetting = {--Revealing Strike Settings
			order = 171,
			name = L["Revealing Strike Settings"],
			type = "header"
		},
		RSBarShow = {--Show Revealing Strike
			order = 172,
			name = L["Show Revealing Strike bar"],
			desc = L["Display the Revealing Strike timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.RS = val
			end,
			get = function(info) return SliceCommander_Save.display.RS end
		},
		PosRS = {--Revealing Strike position
			order = 173,
			name = L["Revealing Strike bar position"],
			desc = L["The place where to display the Revealing Strike bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.RS = val
			end,
			get = function(info) return SliceCommander_Save.Pos.RS end
		},
		headerSDSetting = {--Shadow Dance Settings
			order = 181,
			name = L["Shadow Dance Settings"],
			type = "header"
		},
		SDBarShow = {--Show Shadow Dance
			order = 182,
			name = L["Show Shadow Dance bar"],
			desc = L["Display the Shadow Dance timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.SD = val
				SliceCommander_Save.display.SDCD = val
			end,
			get = function(info) return SliceCommander_Save.display.SD end
		},
		PosSD = {--Show Shadow position
			order = 183,
			name = L["Shadow Dance bar position"],
			desc = L["The place where to display the Shadow Dance bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.SD = val
				SliceCommander_Save.Pos.SDCD = val
			end,
			get = function(info) return SliceCommander_Save.Pos.SD end
		},
		headerHemoSetting = {--Hemorrhage Settings
			order = 191,
			name = L["Hemorrhage Settings"],
			type = "header"
		},
		HemoBarShow = {--Hemorrhage Dance
			order = 192,
			name = L["Show Hemorrhage bar"],
			desc = L["Display the Hemorrhage timer bar"],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.display.HEM = val
			end,
			get = function(info) return SliceCommander_Save.display.HEM end
		},
		PosHemo = {--Hemorrhage position
			order = 193,
			name = L["Hemorrhage bar position"],
			desc = L["The place where to display the Hemorrhage bar"],
			type = "select",
			values = SliceCommander:ArrayPos(),
			set = function(info,val)
				SliceCommander_Save.Pos.HEM = val
			end,
			get = function(info) return SliceCommander_Save.Pos.HEM end
		},
	}
}

local SC_OptionsTable = {
	name = "SliceCommander",
	handler = SliceCommander,
	type = "group",
	args = {--SliceCommander
		IsLocked = {--Lock
			order = 1,
			name = L["Lock"],
			desc = L["Disable Left Click to Move"],
			type = "toggle",
			set = function(info,val)
				SliceCommander:LockCkecked(val)
			end,
			get = function(info) return SliceCommander_Save.IsLocked end
		},
		FullTimer = {--Full timer
			order = 2,
			name = L["Full Timer"],
			desc = L["Make timer full bar duration"],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.FullTimer = val end,
			get = function(info) return SliceCommander_Save.FullTimer end
		},
		PadLatency = {--Latency
			order = 3,
			name = L["Enable Latency"],
			desc = L["Pad Alerts with Latency"],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.PadLatency = val end,
			get = function(info) return SliceCommander_Save.PadLatency end
		},
		SpellText = {--Spell text
			order = 4,
			name = L["Spell Text"],
			desc = L["Enable spell text on bars"],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.SpellText = val end,
			get = function(info) return SliceCommander_Save.SpellText end
		},
		headerDisplaySetting = {--CDisplay Setting
			order = 6,
			name = L["Display Setting"],
			type = "header"
		},
		BarTexture = {--Texture
			order = 7,
			name = L["Bar Texture"],
			desc = L["Bar Texture for timer and energy"],
			type = "select",
			values = function()
				local returnArray = {}
				for key, value in pairs(SliceCommander_BarTextures) do
					returnArray[key] = key
				end
				return returnArray
			end,
			set = function(info,val)
				SliceCommander_Save.BarTexture = val
				SliceCommander:Config_RetextureBars(SliceCommander:BarTexture())
			end,
			get = function(info) return SliceCommander_Save.BarTexture end
		},
		Width = {--Width
			order = 9,
			name = L["Width"],
			desc = L["widthDescription"],
			type = "range",
			min = 125,
			max = 300,
			step = 1,
			bigStep = 5,
			set = function(info,val)
				SliceCommander_Save.Width = val
				if SliceCommander_Save.skin then
					SliceCommander:ApplySkin(false)
				end
				SliceCommander:SetWidth(val)
				if SliceCommander_Save.skin then
					SliceCommander:ApplySkin(true)
				end

			end,
			get = function(info) return SliceCommander_Save.Width end,
			isPercent = false
		},
		EnergyHeight = {--Energy bar height
			order = 10,
			name = L["Energy Height"],
			desc = L["Energy Bar Height"],
			type = "range",
			min = 1,
			max = 40,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				SliceCommander_Save.EnergyHeight = val
				SliceCommander:SetEnergyHeight(val)
			end,
			get = function(info) return SliceCommander_Save.EnergyHeight end,
			isPercent = false
		},
		TimerHeight = {--Timer height
			order = 11,
			name = L["Timer Height"],
			desc = L["Timer Bar Height"],
			type = "range",
			min = 1,
			max = 50,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				SliceCommander_Save.TimerHeight = val
				SliceCommander:SetTimersHeight(val)
				SliceCommander:ApplySkin(SliceCommander_Save.skin)
			end,
			get = function(info) return SliceCommander_Save.TimerHeight end,
			isPercent = false
		},
		BarMargin = {--Margin
			order = 12,
			name = L["Bar Margin"],
			desc = L["Bar Margin"],
			type = "range",
			min = -2,
			max = 10,
			step = 1,
			bigStep = 1,
			set = function(info,val) SliceCommander_Save.BarMargin = val end,
			get = function(info) return SliceCommander_Save.BarMargin end,
			isPercent = false
		},
		FrameOpacity = {--Opacity
			order = 13,
			name = L["Frame opacity"],
			desc = L["Frame opacity"],
			type = "range",
			min = 25,
			max = 100,
			step = 1,
			bigStep = 1,
			set = function(info,val)
				SliceCommander:FrameOpacity(val)
				SliceCommander_Save.FrameOpacity = val
			end,
			get = function(info) return SliceCommander_Save.FrameOpacity end,
			isPercent = false
		},
		headerCheckHealth = {--Check health header
			order = 14,
			name = L["Enable 35% backstab warning"],
			type = "header"
		},
		checkHealthSpec1 = {--Check health spec1
			order = 15,
			name = L["Spec 1"],
			desc = L["Check health for Spec 1"],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.checkHealthSpec1 = val end,
			get = function(info) return SliceCommander_Save.checkHealthSpec1 end
		},
		checkHealthSpec2 = {--Check health spec2
			order = 16,
			name = L["Spec 2"],
			desc = L["Check health for Spec 2"],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.checkHealthSpec2 = val end,
			get = function(info) return SliceCommander_Save.checkHealthSpec2 end
		},
		HealthUnder = {
			order = 17,
			name = L["Sound under 35% health"],
			desc = L["The sound play when the health reach 35% health"],
			type = "select",
			values = function()
				local returnArray = { ["None"] = L["None"] }
				for ignore, SoundName in pairs(SliceCommander_SoundMenu['All']) do
					returnArray[SoundName] = SoundName
				end
				return returnArray
			end,
			set = function(info,val)
				SliceCommander:SoundTest(val)
				SliceCommander_Save.HealthUnder = val
			end,
			get = function(info)
				return SliceCommander_Save.HealthUnder
			end
		},
		headerKick = {
			order = 20,
			name = L["Kick Feature"],
			type = "header"
		},
		kickEnable = {
			order = 21,
			name = L["Enable"],
			desc = L["If enable, a warning message will be display when you succeed to interrupts spellcasting."],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.kickEnable = val end,
			get = function(info) return SliceCommander_Save.kickEnable end
		},
		kickMessage = {
			order = 22,
			name = L["Warning message"],
			desc = L["The warning message which will be display when you kick is successful."],
			type = "input",
			width = "double",
			set = function(info,val) SliceCommander_Save.kickMessage = val end,
			get = function(info) return SliceCommander_Save.kickMessage end
		},
		headerSkin = {
			order = 30,
			name = L["Border Feature"],
			type = "header"
		},
		skinEnable = {
			order = 31,
			name = L["Enable"],
			desc = L["Add a border like the one on Tukui."],
			type = "toggle",
			set = function(info,val)
				SliceCommander_Save.skin = val
				SliceCommander:ApplySkin(val)
				SliceCommander:ShowHideBorder(val)
			end,
			get = function(info) return SliceCommander_Save.skin end
		},
		headerSound = {
			order = 40,
			name = L["Sound settings"],
			type = "header"
		},
		SoundCombat = {--Disable sound out of combat
			order = 41,
			name = L["Sound Combat"],
			desc = L["Disable sounds out of combat"],
			type = "toggle",
			set = function(info,val) SliceCommander_Save.SoundCombat = val end,
			get = function(info) return SliceCommander_Save.SoundCombat end
		},
		SoundChannel = {--SoundChannel
			order = 42,
			name = L["Sound channel"],
			desc = L["Choose the sound channel with which you want to play SliceCommander sounds. 'MASTER' must be use if you want to ear sounds when sound is disable. Other channels should be use if you want o lower volume."],
			type = "select",
			values = function()
				local returnArray = {
					["Master"] = "Master",
					["SFX"] = "SFX",
					["Ambience"] = "Ambience",
					["Music"] = "Music",
				}
				return returnArray
			end,
			set = function(info,val)
				SliceCommander_Save.soundChannel = val
			end,
			get = function(info) return SliceCommander_Save.soundChannel end
		},
		SoundVolume = {--Sound Volume
			order = 43,
			name = L["Sound Volume"],
			desc = L["Warning: It will change the sound channel volume. If other sounds are played with this channel it will also change their volume."],
			type = "range",
			min = 0,
			max = 1,
			step = 0.05,
			bigStep = 0.1,
			set = function(info,val)
				SliceCommander:ChangeVolume(val)
			end,
			get = function(info) return tonumber(SliceCommander:GetVolume()) end,
			isPercent = true
		},
	}
}

function SliceCommander:ChangeVolume(val)
	if SliceCommander_Save.soundChannel == "Master" then
		SetCVar("Sound_MasterVolume",val)
	elseif SliceCommander_Save.soundChannel == "SFX" then
		SetCVar("Sound_SFXVolume",val)
	elseif SliceCommander_Save.soundChannel == "Ambience" then
		SetCVar("Sound_AmbienceVolume",val)
	elseif SliceCommander_Save.soundChannel == "Music" then
		SetCVar("Sound_MusicVolume",val)
	end
end

function SliceCommander:GetVolume()
	if SliceCommander_Save.soundChannel == "Master" then
		return GetCVar("Sound_MasterVolume")
	elseif SliceCommander_Save.soundChannel == "SFX" then
		return GetCVar("Sound_SFXVolume")
	elseif SliceCommander_Save.soundChannel == "Ambience" then
		return GetCVar("Sound_AmbienceVolume")
	elseif SliceCommander_Save.soundChannel == "Music" then
		return GetCVar("Sound_MusicVolume")
	end
end

function SliceCommander:OnInitialize()
	local _, englishClass = UnitClass("player")
	if englishClass == "ROGUE" then
		SliceCommander:RegisterEvent("PLAYER_REGEN_DISABLED")
		SliceCommander:RegisterEvent("PLAYER_REGEN_ENABLED", "PLAYER_REGEN_DISABLED")
		SliceCommander:RegisterEvent("UNIT_COMBO_POINTS")
		SliceCommander:RegisterEvent("PLAYER_TARGET_CHANGED")
		SliceCommander:RegisterEvent("UNIT_HEALTH")
		SliceCommander:RegisterEvent("PLAYER_LOGIN")
		SliceCommander:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	else
		SliceCommander:OnDisable()
		return 0
	end
end

function SliceCommander:InitConfigurationTable()
	AceConfig:RegisterOptionsTable("SliceCommander", SC_OptionsTable)
	AceConfig:RegisterOptionsTable("SliceCommander_DisplayTimer", SC_OptionsTable_DisplayTimer)
	AceConfig:RegisterOptionsTable("SliceCommander_Sound", SC_OptionsTable_Sound)

	local SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander","SliceCommander")
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_DisplayTimer",L["Display settings"],"SliceCommander")
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Sound",L["Sound settings"],"SliceCommander")
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end
end

function SliceCommander:InitFont()
	SC.BarFont1 = SliceCommander:NewFont("VTimerFont2", 8, 0.7, 213/255, 200/255, 184/255)
	SC.BarFont2 = SliceCommander:NewFont("VTimerFont4", 8, 1, 213/255, 200/255, 184/255)
end

function SliceCommander:InitIcon()
	SC.OTHER['BF']['obj'] = SliceCommander:NewIcon("Interface\\Icons\\Ability_Warrior_PunishingBlow")--Create Blade fury icon display when BF is enable
	SC.OTHER['HEALTH']['obj'] = SliceCommander:NewIcon("Interface\\Icons\\Ability_BackStab")--Create Backstab icon display when Target is under 35% HP
end

function SliceCommander:InitPoisonBar()
	SC.BARS['MH']['obj'] = SliceCommander:NewPoisonFrame("BOTTOMLEFT", "TOPLEFT", 0, -3*SC.V.pxH)--Create MH poison timer
	SC.BARS['OH']['obj'] = SliceCommander:NewPoisonFrame("TOPLEFT", "BOTTOMLEFT", 0 , 3*SC.V.pxH)--Create OH poison timer
end

function SliceCommander:DisableIcon(obj, val)
	obj.iconBackground:SetAlpha(0)--Hide icon
	for i,v in pairs(obj.iconBorder) do
		v:Hide()
	end
	obj:SetMinMaxValues(0, val)--Threat value should be between 0 and val
end

function SliceCommander:InitBar()
	SC.OTHER['BG']['obj'] = SliceCommander:BGFrame()--create Bandit's Guile bar
	SC.BARS['CP']['obj'] = SliceCommander:CPFrame()--create Combo Point bar
	SC.BARS['KS']['obj'] = SliceCommander:NewFrame(227/255, 186/255, 132/255)--create Kidney Shot bar
	SC.BARS['CS']['obj'] = SliceCommander:NewFrame(227/255, 186/255, 132/255)--create Cheap Shot bar
	SC.BARS['SnD']['obj'] = SliceCommander:NewFrame(193/255, 79/255, 44/255)--create Slice and Dice bar
	SC.BARS['RUP']['obj'] = SliceCommander:NewFrame(180/255, 38/255, 38/255)--create Rupture bar
	SC.BARS['EA']['obj'] = SliceCommander:NewFrame(151/255, 101/255, 199/255)--create Expose Armor bar
	SC.BARS['GAR']['obj'] = SliceCommander:NewFrame(180/255, 38/255, 38/255)--create Garrote bar
	SC.BARS['REC']['obj'] = SliceCommander:NewFrame(110/255, 172/255, 163/255)--create Recuperate bar
	SC.BARS['VEN']['obj'] = SliceCommander:NewFrame(110/255, 34/255, 45/255)--create Vendetta bar
	SC.BARS['VENCD']['obj'] = SliceCommander:NewFrame(139/255, 78/255, 87/255)--create CD Vendetta bar
	SC.BARS['AR']['obj'] = SliceCommander:NewFrame(226/255, 130/255, 90/255)--create Adrenaline Rush bar
	SC.BARS['ARCD']['obj'] = SliceCommander:NewFrame(255/255, 173/255, 132/255)--create CD Adrenaline Rush bar
	SC.BARS['DP']['obj'] = SliceCommander:NewFrame(64/255, 167/255, 64/255)--create Deadly Poison bar
	SC.BARS['ENV']['obj'] = SliceCommander:NewFrame(64/255, 167/255, 64/255)--create Envenom bar
	SC.BARS['ToT']['obj'] = SliceCommander:NewFrame(77/255, 77/255, 255/255)--create Tricks of Trade bar
	SC.BARS['FE']['obj'] = SliceCommander:NewFrame(37/255, 108/255, 174/255)--create Feint bar
	SC.BARS['FECD']['obj'] = SliceCommander:NewFrame(37/255, 108/255, 174/255)--create Feint CD bar
	SC.BARS['RS']['obj'] = SliceCommander:NewFrame(37/255, 108/255, 174/255)--create Revealing Strike
	SC.BARS['SD']['obj'] = SliceCommander:NewFrame(10/255, 60/255, 80/255)--create Vendetta bar
	SC.BARS['SDCD']['obj'] = SliceCommander:NewFrame(0/255, 100/255, 165/255)--create CD Vendetta bar
	SC.BARS['HEM']['obj'] = SliceCommander:NewFrame(110/255, 34/255, 45/255)--create Hemorrhage bar
	SC.BARS['THREAT']['obj'] = SliceCommander:NewFrame(0, 0, 0)
	SC.BARS['HEALTH']['obj'] = SliceCommander:NewFrame(0, 0, 0)

	SliceCommander:NewTickArray(SC.BARS['RUP'])--Create Ticks marker for Rupture Bar
	SliceCommander:NewTickArray(SC.BARS['GAR'])--Create Ticks marker for Garrote Bar
	SliceCommander:NewTickArray(SC.BARS['REC'])	--Create Ticks marker for Recuperate Bar
	SliceCommander:NewTickArray(SC.BARS['HEM'])--Create Ticks marker for Rupture Bar

	SC.BARS['VENCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Deadliness")
	SC.BARS['ARCD']['obj'].icon:SetTexture("Interface\\Icons\\Spell_Shadow_ShadowWordDominate")
	SC.BARS['ToT']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_TricksOftheTrade")
	SC.BARS['FECD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_feint")
	SC.BARS['SDCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Shadowdance")

	SliceCommander:DisableIcon(SC.BARS['THREAT']['obj'], 110)
	SliceCommander:DisableIcon(SC.BARS['HEALTH']['obj'], 100)
end

function SliceCommander:InitConstantValue()
  	local height, width
	height = tonumber(string.match(GetCVar("gxResolution"), "%d+x(%d+)"))
	width = tonumber(string.match(GetCVar("gxResolution"), "(%d+)x+%d"))
	width = width*SliceCommander_Save.eyefinity--eyefinity is wired
	SC.LastEnergy = UnitMana('player')
	SC.curCombo = 0
	SC.soundRup = {'RUP_Alert', 'RUP_Alert', 'RUP_Alert'}--Initialise sounds array
	SC.soundSnd = {'Tick1', 'Tick2', 'Tick3' }
	SC.RUP_AlertPending = 3
	SC.SND_AlertPending = 3
	SC.V = {
		['bColor'] = {.1,.1,.1},
		['wColor'] = {.6,.6,.6},
		['pxH'] = UIParent:GetHeight() / height,
		['pxW'] = UIParent:GetWidth() / width
	}
	SC.Backdrop = {
		bgFile="Interface\\AddOns\\SliceCommander\\Images\\solid.tga",
		edgeFile="",
		tile=true, tileSize=1, edgeSize=0,
		insets={left=-1*SC.V.pxW, right=-1*SC.V.pxW, top=-1*SC.V.pxH, bottom=-1*SC.V.pxH}
	}
	SC.Spark = {}
end

function SliceCommander:PLAYER_LOGIN()
	SliceCommander:TestValueOptions()
	SliceCommander:InitBarArray()
	SliceCommander:InitConfigurationTable()
	SliceCommander:InitFont()
	SliceCommander:InitConstantValue()
	SliceCommander:CreateCore()
	SliceCommander:InitBar()
	SliceCommander:InitIcon()
	SliceCommander:InitPoisonBar()
	SliceCommander:ApplyConf()
	collectgarbage()
	UpdateAddOnMemoryUsage()
  	print('|cffc14f2cSlice|r|cff40a740Commander|r v'..SC.VERSION..' ('..L["memory use"]..':'..floor(GetAddOnMemoryUsage("SliceCommander"))..' kb)')
end

function SliceCommander:ResetOptions()
	SliceCommander_Save = nil
	SliceCommander:TestValueOptions()
	if SC.BARS['SnD']['obj'] ~= 0 then
		SliceCommander:ApplyConf()
		LibStub("AceConfigRegistry-3.0"):NotifyChange("SliceCommander")
	end
end

function SliceCommander:ShowHideBorder(visible)
	if visible then
		for i,v in pairs(VTimerEnergy.border) do
			v:Show()
		end
		for i,v in pairs(SC.BARS) do
			if i ~= "MH" and i ~= "OH" then
				for j,w in pairs(v['obj'].border) do
					w:Show()
				end
				if i ~= "THREAT" and i ~= "CP" and i ~= "HEALTH" then
					for j,w in pairs(v['obj'].iconBorder) do
						w:Show()
					end
				end
			end
		end
	else
		for i,v in pairs(VTimerEnergy.border) do
			v:Hide()
		end
		for i,v in pairs(SC.BARS) do
			if i ~= "MH" and i ~= "OH" then
				for j,w in pairs(v['obj'].border) do
					w:Hide()
				end
				if i ~= "THREAT" and i ~= "CP" and i ~= "HEALTH" then
					for j,w in pairs(v['obj'].iconBorder) do
						w:Hide()
					end
				end
			end
		end
	end
end

function SliceCommander:ApplyConf()
	SliceCommander:LockCkecked(SliceCommander_Save.IsLocked)
	SliceCommander:SetTimersHeight(SliceCommander_Save.TimerHeight)
	SliceCommander:SetWidth(SliceCommander_Save.Width)
	SliceCommander:SetEnergyHeight(SliceCommander_Save.EnergyHeight)
	SliceCommander:Config_RetextureBars(SliceCommander:BarTexture())
	SliceCommander:FrameOpacity(SliceCommander_Save.FrameOpacity)
	SliceCommander:SetPoisonTransparency(SliceCommander_Save.EnergyTrans)
	SliceCommander:ApplySkin(SliceCommander_Save.skin)
	SliceCommander:ShowHideBorder(SliceCommander_Save.skin)
end

function SliceCommander:ConfigSoundMenuInit(WhichMenu)
	local returnArray = { ["None"] = L["None"] }
	for ignore, SoundName in pairs(SliceCommander_SoundMenu[WhichMenu]) do
		returnArray[SoundName] = SoundName
	end
	return returnArray
end

function SliceCommander:Config_OtherVars()
	SC.Spark[1]:SetPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SliceCommander_Save.Energy1 / UnitManaMax("player") * SliceCommander_Save.Width)-1)*SC.V.pxW, 0)
	SC.Spark[2]:SetPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SliceCommander_Save.Energy2 / UnitManaMax("player") * SliceCommander_Save.Width)-1)*SC.V.pxW, 0)
	SC.Spark[3]:SetPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SliceCommander_Save.Energy3 / UnitManaMax("player") * SliceCommander_Save.Width)-1)*SC.V.pxW, 0)
end

function SliceCommander:LockCkecked(checked)
	if checked then
		SliceCommander:SetPoisonTransparency(SliceCommander_Save.FrameOpacity)
		SliceCommanderFrame:SetAlpha(0)
		SliceCommanderFrame:EnableMouse(false)
		SliceCommander_Save.IsLocked = true
	else
		SliceCommander:SetPoisonTransparency(100)
		SliceCommanderFrame:SetAlpha(.5)
		SliceCommanderFrame:EnableMouse(true)
		SliceCommander_Save.IsLocked = false
	end
end

function SliceCommander:Config_RetextureBars(texture)
	VTimerEnergy:SetStatusBarTexture(texture)
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'CP' then
			v['obj']:SetStatusBarTexture(texture)
		end
	end

	for i = 1, 5 do
		SC.BARS['CP']['obj'].combos[i].bg:SetTexture(texture)
	end
end

function SliceCommander:SetWidth(w)
	local h = SC.BARS['RUP']['obj'].iconBackground:GetHeight()/SC.V.pxH
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'THREAT' and i ~= 'CP' and i ~= "HEALTH" then
			local difW = (w-h-1) - v['obj']:GetWidth()/SC.V.pxW
			v['obj']:SetWidth((w-h-1)*SC.V.pxW)

			if i ~= 'MH' and i ~= 'OH' then
				for y,w in pairs(v['obj'].border) do
					w:SetWidth(w:GetWidth()+difW*SC.V.pxW)
				end
			end
		end
	end
	local difW = h-VTimerEnergy:GetWidth()/SC.V.pxW
	for i,v in pairs(VTimerEnergy.border) do
		v:SetWidth(v:GetWidth()+difW*SC.V.pxW)
	end
	VTimerEnergy:SetWidth(w*SC.V.pxW)
	SC.BARS['MH']['obj']:SetWidth(w*SC.V.pxW)
	SC.BARS['OH']['obj']:SetWidth(w*SC.V.pxW)

	SC.BARS['CP']['obj']:SetWidth(w*SC.V.pxW)
	for i,v in pairs(SC.BARS['CP']['obj'].border) do
		v:SetWidth(v:GetWidth()+difW*SC.V.pxW)
	end

	SC.BARS['THREAT']['obj']:SetWidth(w*SC.V.pxW)
	for i,v in pairs(SC.BARS['THREAT']['obj'].border) do
		v:SetWidth(v:GetWidth()+difW*SC.V.pxW)
	end

	SC.BARS['HEALTH']['obj']:SetWidth(w*SC.V.pxW)
	for i,v in pairs(SC.BARS['HEALTH']['obj'].border) do
		v:SetWidth(v:GetWidth()+difW*SC.V.pxW)
	end

	local cx = 0
	local widthCP = (((w-(2*4))/5))*SC.V.pxW
	for i = 1, 5 do
		SC.BARS['CP']['obj'].combos[i]:ClearAllPoints()
		SC.BARS['CP']['obj'].combos[i]:SetPoint("TOPLEFT", SC.BARS['CP']['obj'], "TOPLEFT", cx, 0)
		SC.BARS['CP']['obj'].combos[i]:SetPoint("BOTTOMRIGHT", SC.BARS['CP']['obj'], "BOTTOMLEFT", cx + widthCP , 0)
		cx = cx + widthCP + 2*SC.V.pxW
	end
	SliceCommander:Config_OtherVars()
end

function SliceCommander:SetSizeBorder(frame, border)
	local k = 6
	for i,v in pairs(border) do
		v:SetHeight(frame:GetHeight()+k*SC.V.pxH)
		v:SetWidth(frame:GetWidth()+k*SC.V.pxW)
		k = k-2
	end
end

function SliceCommander:SetEnergyHeight(h)
	SliceCommanderFrame:SetWidth(h*SC.V.pxW)
	SliceCommanderFrame:SetHeight(h*SC.V.pxH)
	VTimerEnergy:SetHeight(h*SC.V.pxH)
	VTimerEnergyTxt:SetHeight(h*SC.V.pxH)
	SliceCommander_Combo:SetHeight(h*SC.V.pxH)
	SC.Spark[1]:SetHeight(h*SC.V.pxH)
	SC.Spark[2]:SetHeight(h*SC.V.pxH)
	SC.Spark[3]:SetHeight(h*SC.V.pxH)
	SliceCommander:SetSizeBorder(VTimerEnergy, VTimerEnergy.border)
	SliceCommander:PlayerHealth()
end

function SliceCommander:SetTimersHeight(h)
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'CP' and i ~= 'MH' and i ~= 'OH' then
			v['obj']:SetHeight(h*SC.V.pxH)
			v['obj'].text:SetHeight(h*SC.V.pxH)
			v['obj'].text2:SetHeight(h*SC.V.pxH)
			v['obj'].icon:SetHeight(h*SC.V.pxH)
			v['obj'].icon:SetWidth(h*SC.V.pxW)
			v['obj'].iconBackground:SetHeight((h+2)*SC.V.pxH)
			v['obj'].iconBackground:SetWidth((h+2)*SC.V.pxW)

			SliceCommander:SetSizeBorder(v['obj'], v['obj'].border)
			if i ~= 'THREAT' and i ~= 'HEALTH' then
				SliceCommander:SetSizeBorder(v['obj'].icon, v['obj'].iconBorder)
			end
		end
	end

	for i = 1, 3 do
		SC.BARS['REC']['tick']['obj'][i]:SetHeight(h*SC.V.pxH)
	end
	for i = 1, 3 do
		SC.BARS['GAR']['tick']['obj'][i]:SetHeight(h*SC.V.pxH)
	end
	for i = 1, 3 do
		SC.BARS['RUP']['tick']['obj'][i]:SetHeight(h*SC.V.pxH)
	end
	for i = 1, 3 do
		SC.BARS['HEM']['tick']['obj'][i]:SetHeight(h*SC.V.pxH)
	end
	SC.BARS['CP']['obj']:SetHeight(h*SC.V.pxH)
	SC.BARS['CP']['obj'].comboText:SetHeight(h*SC.V.pxH)
	for i = 1, 5 do
		SC.BARS['CP']['obj'].combos[i]:SetHeight((h-2)*SC.V.pxH)
	end
	SliceCommander:SetSizeBorder(SC.BARS['CP']['obj'], SC.BARS['CP']['obj'].border)

	SC.BarFont1:SetFont("Interface\\AddOns\\SliceCommander\\Fonts\\FRIZQT__.TTF", (h-2)*SC.V.pxH)
	SC.BarFont2:SetFont("Interface\\AddOns\\SliceCommander\\Fonts\\FRIZQT__.TTF", (h-2)*SC.V.pxH)
	SliceCommander:SetWidth(SliceCommander_Save.Width)
end

function SliceCommander:FrameOpacity(opValue)
	SliceCommander:SetPoisonTransparency(opValue)

	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil then
			v['obj']:SetAlpha(opValue/100)
		end
	end

	SC.OTHER['BF']['obj']:SetAlpha(opValue/100)
	SC.OTHER['HEALTH']['obj']:SetAlpha(opValue/100)
end

function SliceCommander:TestValueOptions()
	if SliceCommander_Save == nil or SliceCommander_Save.display == nil or SliceCommander_Save.Pos == nil then
		SliceCommander_Save = { display = {}, Pos = {} }
	end
	if SliceCommander_Save.ARSound == nil then
		SliceCommander_Save.ARSound = 'Mario'
	end
	if SliceCommander_Save.ARSound_Glyphed == nil then
		SliceCommander_Save.ARSound_Glyphed = 'Mario glyphed'
	end
	if SliceCommander_Save.IsLocked == nil then
		SliceCommander_Save.IsLocked = true
	end
	if SliceCommander_Save.HideEnergy == nil then
		SliceCommander_Save.HideEnergy = false
	end
	if SliceCommander_Save.Energy1 == nil then
		SliceCommander_Save.Energy1 = 35
	end
	if SliceCommander_Save.Energy2 == nil then
		SliceCommander_Save.Energy2 = 55
	end
	if SliceCommander_Save.Energy3 == nil then
		SliceCommander_Save.Energy3 = 25
	end
	if SliceCommander_Save.Fail == nil then
		SliceCommander_Save.Fail = 'None'
	end
	if SliceCommander_Save.Expire == nil then
		SliceCommander_Save.Expire = 'None'
	end
	if SliceCommander_Save.Tick3 == nil then
		SliceCommander_Save.Tick3 = 'None'
	end
	if SliceCommander_Save.Tick2 == nil then
		SliceCommander_Save.Tick2 = 'None'
	end
	if SliceCommander_Save.Tick1 == nil then
		SliceCommander_Save.Tick1 = 'None'
	end
	if SliceCommander_Save.Applied == nil then
		SliceCommander_Save.Applied = 'None'
	end
	if SliceCommander_Save.Apply3 == nil then
		SliceCommander_Save.Apply3 = 'None'
	end
	if SliceCommander_Save.Apply2 == nil then
		SliceCommander_Save.Apply2 = 'None'
	end
	if SliceCommander_Save.Apply1 == nil then
		SliceCommander_Save.Apply1 = 'None'
	end
	if SliceCommander_Save.EnergySound1 == nil then
		SliceCommander_Save.EnergySound1 = 'None'
	end
	if SliceCommander_Save.EnergySound2 == nil then
		SliceCommander_Save.EnergySound2 = 'None'
	end
	if SliceCommander_Save.EnergySound3 == nil then
		SliceCommander_Save.EnergySound3 = 'None'
	end
	if SliceCommander_Save.HealthUnder == nil then
		SliceCommander_Save.HealthUnder = 'None'
	end
	if SliceCommander_Save.RUP_Fail == nil then
		SliceCommander_Save.RUP_Fail = 'None'
	end
	if SliceCommander_Save.RUP_Expire == nil then
		SliceCommander_Save.RUP_Expire = 'None'
	end
	if SliceCommander_Save.RUP_Applied == nil then
		SliceCommander_Save.RUP_Applied = 'None'
	end
	if SliceCommander_Save.RUP_Refresh3 == nil then
		SliceCommander_Save.RUP_Refresh3 = 'None'
	end
	if SliceCommander_Save.RUP_Refresh2 == nil then
		SliceCommander_Save.RUP_Refresh2 = 'None'
	end
	if SliceCommander_Save.RUP_Refresh1 == nil then
		SliceCommander_Save.RUP_Refresh1 = 'None'
	end
	if SliceCommander_Save.RUP_Alert == nil then
		SliceCommander_Save.RUP_Alert = 'None'
	end
	if SliceCommander_Save.MidHealthUnder == nil then
		SliceCommander_Save.MidHealthUnder = 'None'
	end
	if SliceCommander_Save.LowHealthUnder == nil then
		SliceCommander_Save.LowHealthUnder = 'None'
	end
	if SliceCommander_Save.BGSound == nil then
		SliceCommander_Save.BGSound = 'None'
	end
	if SliceCommander_Save.Width == nil then
		SliceCommander_Save.Width = 279
	end
	if SliceCommander_Save.PadLatency == nil then
		SliceCommander_Save.PadLatency = true
	end
	if SliceCommander_Save.SpellText == nil then
		SliceCommander_Save.SpellText = true
	end
	if SliceCommander_Save.FullTimer == nil then
		SliceCommander_Save.FullTimer = false
	end
	if SliceCommander_Save.SoundCombat == nil then
		SliceCommander_Save.SoundCombat = true
	end
	if SliceCommander_Save.EnergyTrans == nil then
		SliceCommander_Save.EnergyTrans = 0
	end
	if SliceCommander_Save.BarMargin == nil then
		SliceCommander_Save.BarMargin = 7
	end
	if SliceCommander_Save.PoisonTransparencyShow == nil then
		SliceCommander_Save.PoisonTransparencyShow = true
	end
	if SliceCommander_Save.checkHealthSpec1 == nil then
		SliceCommander_Save.checkHealthSpec1 = true
	end
	if SliceCommander_Save.checkHealthSpec2 == nil then
		SliceCommander_Save.checkHealthSpec2 = true
	end
	if SliceCommander_Save.ThreatGroupOnly == nil then
		SliceCommander_Save.ThreatGroupOnly = true
	end
	if SliceCommander_Save.EnergyHeight == nil then
		SliceCommander_Save.EnergyHeight = 28
	end
	if SliceCommander_Save.TimerHeight == nil then
		SliceCommander_Save.TimerHeight = 17
	end
	if SliceCommander_Save.BarTexture == nil then
		SliceCommander_Save.BarTexture = 'Tukui'
	end
	if SliceCommander_Save.display.CP == nil then
		SliceCommander_Save.display.CP = true
	end
	if SliceCommander_Save.display.Poison == nil then
		SliceCommander_Save.display.Poison = true
	end
	if SliceCommander_Save.display.DP == nil then
		SliceCommander_Save.display.DP = false
	end
	if SliceCommander_Save.display.ENV == nil then
		SliceCommander_Save.display.ENV = true
	end
	if SliceCommander_Save.display.EA == nil then
		SliceCommander_Save.display.EA = true
	end
	if SliceCommander_Save.display.SnD == nil then
		SliceCommander_Save.display.SnD = true
	end
	if SliceCommander_Save.display.VEN == nil then
		SliceCommander_Save.display.VEN = true
	end
	if SliceCommander_Save.display.VENCD == nil then
		SliceCommander_Save.display.VENCD = true
	end
	if SliceCommander_Save.display.SD == nil then
		SliceCommander_Save.display.SD = true
	end
	if SliceCommander_Save.display.HEM == nil then
		SliceCommander_Save.display.HEM = true
	end
	if SliceCommander_Save.display.SDCD == nil then
		SliceCommander_Save.display.SDCD = true
	end
	if SliceCommander_Save.display.REC == nil then
		SliceCommander_Save.display.REC = true
	end
	if SliceCommander_Save.display.KS == nil then
		SliceCommander_Save.display.KS = true
	end
	if SliceCommander_Save.display.CS == nil then
		SliceCommander_Save.display.CS = true
	end
	if SliceCommander_Save.display.ToT == nil then
		SliceCommander_Save.display.ToT = true
	end
	if SliceCommander_Save.display.RUP == nil then
		SliceCommander_Save.display.RUP = true
	end
	if SliceCommander_Save.display.GAR == nil then
		SliceCommander_Save.display.GAR = true
	end
	if SliceCommander_Save.display.THREAT == nil then
		SliceCommander_Save.display.THREAT = true
	end
	if SliceCommander_Save.display.AR == nil then
		SliceCommander_Save.display.AR = true
	end
	if SliceCommander_Save.display.ARCD == nil then
		SliceCommander_Save.display.ARCD = true
	end
	if SliceCommander_Save.display.BF == nil then
		SliceCommander_Save.display.BF = true
	end
	if SliceCommander_Save.display.FE == nil then
		SliceCommander_Save.display.FE = true
	end
	if SliceCommander_Save.display.HEALTH == nil then
		SliceCommander_Save.display.HEALTH = false
	end
	if SliceCommander_Save.display.RS == nil then
		SliceCommander_Save.display.RS = false
	end
	if SliceCommander_Save.Pos.CP == nil then
		SliceCommander_Save.Pos.CP = -1
	end
	if SliceCommander_Save.Pos.THREAT == nil then
		SliceCommander_Save.Pos.THREAT = -2
	end
	if SliceCommander_Save.Pos.CS == nil then
		SliceCommander_Save.Pos.CS = -3
	end
	if SliceCommander_Save.Pos.KS == nil then
		SliceCommander_Save.Pos.KS = -4
	end
	if SliceCommander_Save.Pos.HEALTH == nil then
		SliceCommander_Save.Pos.HEALTH = -5
	end
	if SliceCommander_Save.Pos.REC == nil then
		SliceCommander_Save.Pos.REC = 1
	end
	if SliceCommander_Save.Pos.SnD == nil then
		SliceCommander_Save.Pos.SnD = 2
	end
	if SliceCommander_Save.Pos.RUP == nil then
		SliceCommander_Save.Pos.RUP = 3
	end
	if SliceCommander_Save.Pos.GAR == nil then
		SliceCommander_Save.Pos.GAR = 4
	end
	if SliceCommander_Save.Pos.ToT == nil then
		SliceCommander_Save.Pos.ToT = 5
	end
	if SliceCommander_Save.Pos.VEN == nil then
		SliceCommander_Save.Pos.VEN = 6
	end
	if SliceCommander_Save.Pos.VENCD == nil then
		SliceCommander_Save.Pos.VENCD = 6
	end
	if SliceCommander_Save.Pos.SD == nil then
		SliceCommander_Save.Pos.SD = 6
	end
	if SliceCommander_Save.Pos.SDCD == nil then
		SliceCommander_Save.Pos.SDCD = 6
	end
	if SliceCommander_Save.Pos.DP == nil then
		SliceCommander_Save.Pos.DP = 7
	end
	if SliceCommander_Save.Pos.ENV == nil then
		SliceCommander_Save.Pos.ENV = 8
	end
	if SliceCommander_Save.Pos.AR == nil then
		SliceCommander_Save.Pos.AR = 9
	end
	if SliceCommander_Save.Pos.ARCD == nil then
		SliceCommander_Save.Pos.ARCD = 9
	end
	if SliceCommander_Save.Pos.EA == nil then
		SliceCommander_Save.Pos.EA = 10
	end
	if SliceCommander_Save.Pos.FE == nil then
		SliceCommander_Save.Pos.FE = 11
	end
	if SliceCommander_Save.Pos.FECD == nil then
		SliceCommander_Save.Pos.FECD = 11
	end
	if SliceCommander_Save.Pos.RS == nil then
		SliceCommander_Save.Pos.RS = 12
	end
	if SliceCommander_Save.Pos.HEM == nil then
		SliceCommander_Save.Pos.HEM = 12
	end
	if SliceCommander_Save.FrameOpacity == nil then
		SliceCommander_Save.FrameOpacity = 100
	end
	if SliceCommander_Save.ARMarioMod == nil then
		SliceCommander_Save.ARMarioMod = false
	end
	if SliceCommander_Save.kickEnable == nil then
		SliceCommander_Save.kickEnable = true
	end
	if SliceCommander_Save.kickMessage == nil then
		SliceCommander_Save.kickMessage = "Kick successful"
	end
	if SliceCommander_Save.point == nil then
		SliceCommander_Save.point = "TOP"
	end
	if SliceCommander_Save.relativeTo == nil then
		SliceCommander_Save.relativeTo = "UIParent"
	end
	if SliceCommander_Save.relativePoint == nil then
		SliceCommander_Save.relativePoint = "TOP"
	end
	if SliceCommander_Save.xOfs == nil then
		SliceCommander_Save.xOfs = 0
	end
	if SliceCommander_Save.yOfs == nil then
		SliceCommander_Save.yOfs = -300
	end
	if SliceCommander_Save.skin == nil then
		SliceCommander_Save.skin = true
	end
	if SliceCommander_Save.eyefinity == nil then
		SliceCommander_Save.eyefinity = 1
	end
	if SliceCommander_Save.BanditGuild == nil then
		SliceCommander_Save.BanditGuild = true
	end
	if SliceCommander_Save.healthValue == nil then
		SliceCommander_Save.healthValue = false
	end
	if SliceCommander_Save.healthHideCombat == nil then
		SliceCommander_Save.healthHideCombat = true
	end
	if SliceCommander_Save.soundChannel == nil then
		SliceCommander_Save.soundChannel = "Master"
	end
end